/*
 * Copyright 2002-2005 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sf.ldaptemplate.support.acegi;

import org.acegisecurity.Authentication;
import org.acegisecurity.context.SecurityContextHolder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import net.sf.ldaptemplate.AuthenticationSource;

/**
 * An AuthenticationSource to retrieve authentication information stored in
 * Acegi's SecurityContextHolder. It is possible to use this in two different
 * ways. Either the full user DN is placed as the principal in the
 * Authentication object a modification of Acegi's own
 * LdapAuthenticationProvider can be used. In order to make sure that sufficient
 * information is present in the Acegi Authentication object (specifically, the
 * distinguished name of the logged in user needs to be present),
 * {@link net.sf.ldaptemplate.support.acegi.LdapTemplateLdapAuthenticationProvider}
 * needs to be used as AuthenticationProvider in stead of the standard
 * LdapAuthenticationProvider.
 * 
 * @author Mattias Arthursson
 * 
 */
public class AcegiAuthenticationSource implements AuthenticationSource {
    private static final Log log = LogFactory
            .getLog(AcegiAuthenticationSource.class);

    /**
     * Get the principals of the logged in user, in this case the distinguished
     * name.
     * 
     * @return the distinguished name of the logged in user.
     */
    public String getPrincipal() {
        Authentication authentication = SecurityContextHolder.getContext()
                .getAuthentication();
        if (authentication != null) {
            Object principal = authentication.getPrincipal();
            if (!(principal instanceof LdapTemplateUser)) {
                throw new IllegalArgumentException(
                        "The principal property of the authentication object -"
                                + "needs to be a LdapTemplateUser."
                                + "Are you using LdapTemplateLdapAuthenticationProvider as required?");
            } else {
                LdapTemplateUser details = (LdapTemplateUser) principal;
                return details.getDn();
            }
        } else {
            log.warn("No Authentication object set in SecurityContext - "
                    + "returning empty String as Principal");
            return "";
        }
    }

    /*
     * @see net.sf.ldaptemplate.AuthenticationSource#getCredentials()
     */
    public String getCredentials() {
        Authentication authentication = SecurityContextHolder.getContext()
                .getAuthentication();

        if (authentication != null) {
            return (String) authentication.getCredentials();
        } else {
            log.warn("No Authentication object set in SecurityContext - "
                    + "returning empty String as Credentials");
            return "";
        }
    }

}
